@@ -27,10 +27,18 @@ from glance import scrubber
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
+CONF.import_opt('use_user_token', 'glance.registry.client')
 
 RESTRICTED_URI_SCHEMAS = frozenset(['file', 'filesystem', 'swift+config'])
 
 
+def check_reserved_stores(enabled_stores):
+    for store in enabled_stores:
+        if store.startswith("os_glance_"):
+            return True
+    return False
+
+
 def safe_delete_from_backend(context, image_id, location):
     """
     Given a location, delete an image from the store and
@@ -45,7 +53,15 @@ def safe_delete_from_backend(context, image_id, location):
     """
 
     try:
-        ret = store_api.delete_from_backend(location['url'], context=context)
+        if CONF.enabled_backends:
+            backend = location['metadata'].get('store')
+            ret = store_api.delete(location['url'],
+                                   backend,
+                                   context=context)
+        else:
+            ret = store_api.delete_from_backend(location['url'],
+                                                context=context)
+
         location['status'] = 'deleted'
         if 'id' in location:
             db_api.get_api().image_location_delete(context, image_id,
@@ -132,5 +148,57 @@ def validate_external_location(uri):
     # TODO(zhiyan): This function could be moved to glance_store.
     # TODO(gm): Use a whitelist of allowed schemes
     scheme = urlparse.urlparse(uri).scheme
-    return (scheme in store_api.get_known_schemes() and
+    known_schemes = store_api.get_known_schemes()
+    if CONF.enabled_backends:
+        known_schemes = store_api.get_known_schemes_for_multi_store()
+
+    return (scheme in known_schemes and
             scheme not in RESTRICTED_URI_SCHEMAS)
+
+
+def _get_store_id_from_uri(uri):
+    scheme = urlparse.urlparse(uri).scheme
+    location_map = store_api.location.SCHEME_TO_CLS_BACKEND_MAP
+    url_matched = False
+    if scheme not in location_map:
+        LOG.warning("Unknown scheme '%(scheme)s' found in uri '%(uri)s'", {
+            'scheme': scheme, 'uri': uri})
+        return
+    for store in location_map[scheme]:
+        store_instance = location_map[scheme][store]['store']
+        url_prefix = store_instance.url_prefix
+        if url_prefix and uri.startswith(url_prefix):
+            url_matched = True
+            break
+
+    if url_matched:
+        return store
+    else:
+        LOG.warning("Invalid location uri %s", uri)
+        return
+
+
+def update_store_in_locations(locations, image_id):
+    for loc in locations:
+        store_id = _get_store_id_from_uri(loc['url'])
+        if store_id:
+            if 'store' in loc['metadata']:
+                old_store = loc['metadata']['store']
+                if old_store != store_id:
+                    LOG.debug("Store '%(old)s' has changed to "
+                              "'%(new)s' by operator, updating "
+                              "the same in the location of image "
+                              "'%(id)s'", {'old': old_store,
+                                           'new': store_id,
+                                           'id': image_id})
+
+            loc['metadata']['store'] = store_id
+
+
+def get_dir_separator():
+    separator = ''
+    staging_dir = "file://%s" % getattr(
+        CONF, 'os_glance_staging_store').filesystem_store_datadir
+    if not staging_dir.endswith('/'):
+        separator = '/'
+    return separator, staging_dir
